<?php
/**
*   Update checker widget for PHP-Gtk
*   @author Christian Weiske <cweiske@cweiske.de>
*
*   Displays information the UpdateChecker class
*   provides
*
*   requires the UpdateChecker class by Christian Weiske
*/

require_once( 'UpdateChecker.inc');

class Gtk_UpdateChecker
{
    /**
    *   The widget to add to your code
    */
    var $widget;
    
    var $bShowInterface = true;
    
    var $arWidgets      = array();
    
    var $strXsaUrl      = null;
    var $strProductId   = null;
    var $strVersion     = null;
    var $strDate        = null;
    
    var $bInterfaceCreated  = false;
    
    /**
    *   The UpdateChecker object
    */
    var $objChecker     = null;
    
    
    /**
    *   constructor
    */
    function Gtk_UpdateChecker( $strProductName, $strXsaUrl, $strProductId, $strVersion = null, $strDate = null)
    {
        strstr( PHP_OS, 'WIN') ? $this->strPathSeparator = '\\' : $this->strPathSeparator = '/';
        $this->setProductInformation( $strProductName, $strXsaUrl, $strProductId, $strVersion, $strDate);
    }//function Gtk_UpdateChecker( $strXsaUrl, $strProductId, $strVersion = null, $strDate = null)
    
    
    
    /**
    *   updates the product information
    */
    function setProductInformation( $strProductName, $strXsaUrl, $strProductId, $strVersion = null, $strDate = null)
    {
        $this->strProductName = $strProductName;
        $this->strXsaUrl      = $strXsaUrl;
        $this->strProductId   = $strProductId;
        $this->strVersion     = $strVersion;
        $this->strDate        = $strDate;
    }//function setProductInformation( $strXsaUrl, $strProductId, $strVersion = null, $strDate = null)    
    
    
    /**
    *   creates the widgets for the interface
    */
    function createInterface()
    {
        $glade = &new GladeXML( dirname( __FILE__) . $this->strPathSeparator . "Gtk_UpdateChecker.glade", 'ntbkUpdates');
        foreach( array(
            'ntbkUpdates', 'btnCheck', 'lblCheckInformation', 'lblCheckVersion', 'lblChecking', 'lblNoUpdate', 'btnUpdateAvailable',
            'lblApplicationName','lblInstalledVersion','lblNewVersion','lblReleaseDate','lblChanges','txtHomepage','txtDownload',
            'lblDownloadsize','btnGoHomepage','btnGoDownload','lblError'
            ) as $strWidget)
        {
            $this->arWidgets[$strWidget] = $glade->get_widget( $strWidget);
        }
        
        $this->widget   = $this->arWidgets['ntbkUpdates'];
        
        $this->arWidgets['btnCheck']          ->connect_object( 'clicked', array( &$this, 'checkForUpdate'));
        $this->arWidgets['btnUpdateAvailable']->connect_object( 'clicked', array( &$this, 'showUpdateInformation'));
        
        $this->arWidgets['ntbkUpdates']->set_page( 0);
        
        $this->bInterfaceCreated    = true;
        $this->displayProductInformation();
    }//function createInterface()
    
    
    
    /**
    *   displays name and current version of the product
    */    
    function displayProductInformation()
    {
        if( !$this->bInterfaceCreated) { return; }
        
        $this->arWidgets['lblCheckInformation'] ->set_text( $this->strProductName);
        $this->arWidgets['lblApplicationName']  ->set_text( $this->strProductName);
        
        if( $this->strVersion !== null) {
            $this->arWidgets['lblCheckVersion']     ->set_text( $this->strVersion);
            $this->arWidgets['lblInstalledVersion'] ->set_text( $this->strVersion);
        } else {
            $this->arWidgets['lblCheckVersion']     ->set_text( $this->strDate);
            $this->arWidgets['lblInstalledVersion'] ->set_text( $this->strDate);
        }
    }//function displayProductInformation()
    
    
    
    /**
    *   checks if an update is available
    */
    function checkForUpdate()
    {
        $this->arWidgets['ntbkUpdates']->set_page( 0);
        $this->arWidgets['btnCheck']->hide();
        $this->arWidgets['lblChecking']->show();
        while(gtk::events_pending()) gtk::main_iteration();
        
        $this->objChecker   = new UpdateChecker();
        //this may take some time:
        $bUpdateAvailable   = $this->objChecker->isUpdateAvailable( $this->strXsaUrl, $this->strProductId, $this->strVersion, $this->strDate);
        
        while(gtk::events_pending()) gtk::main_iteration();
        
        $this->arWidgets['lblChecking']->hide();
        if( $bUpdateAvailable) {
            //$this->arWidgets['btnUpdateAvailable']->show();
            $this->showUpdateInformation();
        } else {
            //no update
            $this->arWidgets['lblNoUpdate']->show();
        }
        while(gtk::events_pending()) gtk::main_iteration();
        
        //perhaps we had one
        $this->displayError();
    }//function checkForUpdate()
    
    
    
    /**
    *   displays the update information
    *   toggles the notebook to the second tab
    */
    function showUpdateInformation()
    {
        $arInformation  = $this->objChecker->getUpdateInformation( $this->strXsaUrl, $this->strProductId);
        
        if( $arInformation === false)
        {   //error - what to do?
            $this->displayError();
            return;
        }
        
        $this->arWidgets['lblNewVersion']  ->set_text( $arInformation['version']);
        $this->arWidgets['lblReleaseDate'] ->set_text( $arInformation['last-release']);
        
        if( !isset( $arInformation['changes'])) { $arInformation['changes'] = ''; }
        $this->arWidgets['lblChanges']     ->set_text( $arInformation['changes']);
        
        if( !isset( $arInformation['info-url'])) { $arInformation['info-url'] = ''; }
        $this->arWidgets['txtHomepage']    ->set_text( $arInformation['info-url']);
        
        if( !isset( $arInformation['download-url'])) { $arInformation['download-url'] = ''; }
        $this->arWidgets['txtDownload']    ->set_text( $arInformation['download-url']);
        
        if( !isset( $arInformation['download-size'])) { $arInformation['download-size'] = ''; }
        $this->arWidgets['lblDownloadsize']->set_text( $arInformation['download-size']);
    
        $this->arWidgets['ntbkUpdates']->set_page( 1);
        
        while(gtk::events_pending()) gtk::main_iteration();
    }//function showUpdateInformation()
    
    
    
    /**
    *   displays an error message if there is an error
    *   with the UpdateChecker
    */
    function displayError()
    {
        if( $this->objChecker === null || !$this->objChecker->hadError()) {
            return;
        }
        
        //we've got an error
        $this->arWidgets['lblError']   ->set_text( $this->objChecker->getLastError());
        $this->arWidgets['ntbkUpdates']->set_page( 2);
        while(gtk::events_pending()) gtk::main_iteration();
    }//function displayError()
    
    
    
}//class Gtk_UpdateChecker
?>